from stat import S_ISFIFO
from os import access, mkdir, stat, R_OK
from os.path import exists

from pipe_io_server import PipeReaderServer, PipeWriterServer

from tfw.internals.inotify import InotifyObserver, InotifyFileCreatedEvent, InotifyFileDeletedEvent


class PipeConnector:
    def __init__(self, path):
        self.recv_pipes, self.send_pipes = {}, {}
        self.observer = self.build_observer(path)
        self.observer.on_any_event = self._on_any_event
        self.observer.start()

    def build_observer(self, path): # pylint: disable=no-self-use
        if not exists(path):
            mkdir(path)
        if not access(path, R_OK):
            raise ValueError('Path does not exist or is not accessible.')
        observer = InotifyObserver(path, patterns=['*send*', '*recv*'])
        return observer

    def _on_any_event(self, event):
        path = event.src_path
        if isinstance(event, InotifyFileCreatedEvent) and self._is_pipe(path):
            self._create_pipe(path)
        elif isinstance(event, InotifyFileDeletedEvent):
            self._delete_pipe(path)

    @staticmethod
    def _is_pipe(path):
        return exists(path) and S_ISFIFO(stat(path).st_mode)

    def _create_pipe(self, path):
        if self._find_pipe(path):
            return
        server = None
        if 'recv' in path:
            pipes, server = self.recv_pipes, self.build_writer(path)
        elif 'send' in path:
            pipes, server = self.send_pipes, self.build_reader(path)
        if server:
            server.start()
            pipes[path] = server

    def _find_pipe(self, path):
        if path in self.recv_pipes.keys():
            return self.recv_pipes
        if path in self.send_pipes.keys():
            return self.send_pipes
        return None

    def build_reader(self, path): # pylint: disable=no-self-use
        return PipeReaderServer(path, manager_pipes=False)

    def build_writer(self, path): # pylint: disable=no-self-use
        return PipeWriterServer(path, manage_pipes=False)

    def _delete_pipe(self, path):
        pipes = self._find_pipe(path)
        if pipes:
            pipes[path].stop()
            del pipes[path]

    def broadcast(self, message):
        for _, server in self.recv_pipes.items():
            server.send_message(message)
